evm.namespace('evm.util');
(function () {
    /**
     * 提示框组件
     * @desc
     * <pre>
     * v1.1.0 2017-11-06
     * 1.添加属性autoClose，用于配置是否自动隐藏提示框
     *
     * v1.0.0 2017-04-27
     * 1.修复提示框按确定键隐藏嵌套提示框的Bug
     * </pre>
     * @version 1.0.0
     * @author wuqq@evmtv.com
     * @param {Object} [options] 组件的配置对象
     * @param {String} [options.boxId] 提示框的容器DOM的id属性值
     * @param {String} [options.title] 提示框的标题，默认值为“提示”
     * @param {String} [options.msg] 提示框的文字提示，默认为空
     * @param {Array} [options.buttons] 提示框的操作按钮配置数组，默认为空数组
     * <pre>
     * 默认
     * buttons = []
     * //buttons有两个成员
     * //成员名字可自定义
     * buttons = [
     *      {
     *          name:'确定',
     *          callback:function(){
     *              //定义操作按钮的回调函数
     *              //即按确定键执行该函数
     *              //其他按钮类似
     *          }
     *      },
     *      {
     *          name:'取消'
     *      }
     * ]
     * //buttons只有一个成员
     * buttons = [
     *      {
     *          name:'确定'
     *      }
     * ]
     * </pre>
     * @param {Number} [options.timeout] 如果没有操作按钮，提示框面板经过timeout秒自动隐藏
     * @param {Number} [options.buttonsActiveIndex] 有操作按钮时，按钮焦点位置
     * @param {Boolean} [options.isVisible] 提示框面板显示或者隐藏
     * @example
     * //页面需要引用css
     * /swatapps/ape/util/tipbox/skin/css/main.min.css
     * 如果提示框样式需要定制化，请在项目中覆盖原始的css样式
     *
     * //页面需要引用js
     * /swatapps/ape/util/tipbox/main.min.js
     *
     * //定义一个提示框对象
     * var tipbox = new evm.util.Tipbox({
     *      buttons:[
     *          {
     *              name:'确认',
     *              callback:function(){
     *                  //按键确认操作
     *              }
     *          }
     *      ]
     * });
     * //提示框组件的初始化方法
     * tipbox.init();
     * //更新和显示提示框面板
     * tipbox.updatePanel('提示','确认取消关注吗？',[
     *      {
     *          name:'确认',
     *          callback:function(){
     *              //点击确认按钮执行
     *          }
     *      },
     *      {
     *          name:'取消',
     *          callback:function(){
     *              //点击取消按钮执行
     *          }
     *      }
     * ]);
     * @constructor
     */
    evm.util.Tipbox = function (options) {
        var defaults = {
            /*基类配置项*/
            prefix: 'Ape',
            compName: 'tipbox',
            parentComp: null,
            /*本身的配置项*/
            boxId: '',
            title: '提示',
            msg: '',
            buttons: [],
            timeout: 5,
            buttonsActiveIndex: 0,
            isVisible: false,
            autoClose: false
        };

        var config = evm.util.common.extend(defaults, options);
        evm.ext.comp.BaseComponent.call(this, config);

        //保存原始配置
        this._config = evm.util.common.extend(true, {}, config);
        evm.util.common.extend(this, config);

        this._timer = -1;

        this._hasNestTipBox = false;
    }
    evm.util.Tipbox.prototype = {
        constructor: evm.util.Tipbox,
        _grabEvent: function (_v, _e) {
            var isBlock = 0;
            switch (_e.alias) {
                case 'RIGHT':
                    this._onChange(1);
                    break;
                case 'LEFT':
                    this._onChange(-1);
                    break;
                case 'SELECT':
                    this._onSelect();
                    break;
            }
            return isBlock;
        },
        _addListener: function () {
            evm.controller.subscribe({
                type: 0,
                method: 'add',
                callback: this._grabEvent,
                context: this
            });
        },
        _removeListener: function () {
            evm.controller.subscribe({
                type: 1,
                method: 'remove',
                callback: this._grabEvent,
                context: this
            });
        },
        _onChange: function (step) {
            var targetActiveIndex = this.buttonsActiveIndex + step;
            targetActiveIndex = targetActiveIndex > 0 ? targetActiveIndex : targetActiveIndex + this.buttons.length;
            targetActiveIndex %= this.buttons.length;

            if (!isNaN(targetActiveIndex)) {
                this._onBlur();
                this.buttonsActiveIndex = targetActiveIndex;
                this._onFocus();

            }
        },
        _onBlur: function () {
            var len;
            if (len = this.buttons.length) {
                var buttonElem = evm.getElementsByClassName('button-' + len + '-' + this.buttonsActiveIndex, this._boxElem, 'div')[0];
                evm.removeClass(buttonElem, 'focus');
            }
        },
        _onFocus: function () {
            var len;
            if (len = this.buttons.length) {
                var buttonElem = evm.getElementsByClassName('button-' + len + '-' + this.buttonsActiveIndex, this._boxElem, 'div')[0];
                evm.addClass(buttonElem, 'focus');
            }
        },
        _onSelect: function () {
            if (this.buttons && this.buttons.length) {
                var button = this.buttons[this.buttonsActiveIndex];
                if (button.callback) {
                    button.callback.call(this);
                }
            }
            if (!this._hasNestTipBox) {
                this._hide();
            }
            this.setHasNestTipBox(false);
        },
        /**
         * 初始化容器DOM元素
         * @param callback
         * @private
         */
        _createDom: function (callback) {
            var contentTpl = '<div class="title"></div>' +
                '<div class="msg"></div>' +
                '<div class="buttonWrapper"></div>';
            var containerElm = evm.$('container') || document.getElementsByTagName('body')[0];

            this.boxId = this.boxId || this.comp_name;
            this._boxElem = evm.$(this.boxId);
            //生成容器DOM
            if (!this._boxElem) {
                this._boxElem = evm.createElem('div', {
                    id: this.boxId,
                    className: this.class_name
                }, containerElm);

                evm.html(this._boxElem, contentTpl);

                evm.css(this._boxElem, {
                    display: this.isVisible ? 'block' : 'none'
                });
                callback();
            }
        },
        /**
         * 提示框初始化
         */
        init: function () {
            this._createDom(evm.util.common.proxy(this, function () {
                this._updateTitle();
                this._updateMsg();
                this._updateButtons();
            }));
        },
        /**
         * 更新提示框和自动显示面板
         * @param {String} [title] 标题
         * @param {String} [msg] 提示信息
         * @param {Array} [buttons] 操作按钮数组 {@link options.buttons}
         */
        updatePanel: function (title, msg, buttons) {
            this._updateTitle(title);
            this._updateMsg(msg);
            this._updateButtons(buttons);

            this._show();
        },
        /**
         * 更新标题
         * @param {String} [newTitle] 标题
         * @private
         */
        _updateTitle: function (newTitle) {
            if (newTitle) {
                this.title = newTitle;
            }

            var titleElem = evm.getElementsByClassName('title', this._boxElem, 'div')[0];
            evm.html(titleElem, this.title);
        },
        /**
         * 更新提示内容
         * @param {String} [newMsg] 新提示内容
         * @private
         */
        _updateMsg: function (newMsg) {
            if (newMsg) {
                this.msg = newMsg;
            }

            var msgElem = evm.getElementsByClassName('msg', this._boxElem, 'div')[0];
            evm.html(msgElem, this.msg);
        },
        /**
         * 更新按钮组
         *  @param {Array} [newButtons] 新的操作按钮配置数组 {@link options.buttons}
         *  @private
         */
        _updateButtons: function (newButtons) {
            var buttonWrapperElem = evm.getElementsByClassName('buttonWrapper', this._boxElem, 'div')[0],
                buttonsHtml = '',
                i = 0,
                len;

            buttonWrapperElem.innerHTML = '';

            if (newButtons) {
                this.buttons = newButtons;
            }

            for (len = this.buttons.length; i < len; i++) {
                buttonsHtml += '<div class="button-' + len + '-' + i + '">' + this.buttons[i].name + '</div>';
            }
            evm.html(buttonWrapperElem, buttonsHtml);
        },
        /**
         * 显示提示面板
         * @private
         */
        _show: function () {
            //去掉延时执行的函数
            if (this._timer > 0) {
                clearTimeout(this._timer);
            }
            if (!this.isVisible) {
                this._addListener();
            }

            if (this.buttons.length) {
                this._onFocus();
            }
            else if (this.autoClose) {
                this._timer = setTimeout(evm.util.common.proxy(this, function () {
                    this._hide();
                }), this.timeout * 1000);
            }

            this.isVisible = true;
            evm.util.common.show(this._boxElem);
        },
        /**
         * 隐藏提示面板
         * @private
         */
        _hide: function () {
            this._removeListener();

            this.isVisible = false;
            evm.util.common.hide(this._boxElem);
        },
        /**
         * 隐藏提示面板
         */
        hide: function () {
            this._hide();
        },
        setAutoClose: function (autoClose) {
            this.autoClose = !!autoClose;
        },
        /**
         * 设置没有按钮操作时，提示框自动隐藏的时间，单位为s
         * @param {Number} timer
         */
        setTimer: function (timer) {
            timer = +timer;
            if (timer >= 0) {
                this.timeout = timer;
            }
        },
        /**
         * 设置是否有嵌套的tipbox
         * @param {boolean} hasNest
         */
        setHasNestTipBox: function (hasNest) {
            this._hasNestTipBox = hasNest;
        }
    };
})();